@@ -40,15 +40,15 @@ import time
 
 # only needed for arc
 try:
-  from authres import AuthenticationResultsHeader
-except:
-  pass
+    from authres import AuthenticationResultsHeader
+except ImportError:
+    pass
 
 # only needed for ed25519 signing/verification
 try:
     import nacl.signing
     import nacl.encoding
-except:
+except ImportError:
     pass
 
 from dkim.canonicalization import (
@@ -67,10 +67,10 @@ from dkim.crypto import (
     UnparsableKeyError,
     )
 try:
-  from dkim.dnsplug import get_txt
-except:
-  def get_txt(s):
-    raise RuntimeError("DKIM.verify requires DNS or dnspython module")
+    from dkim.dnsplug import get_txt
+except ImportError:
+    def get_txt(s):
+        raise RuntimeError("DKIM.verify requires DNS or dnspython module")
 from dkim.util import (
     get_default_logger,
     InvalidTagValueList,
@@ -352,7 +352,18 @@ def fold(header, namelen=0):
     ' foo'
     >>> len(fold(b'foo'*25).splitlines()[0])
     72
+    >>> text(fold(b'x'))
+    'x'
+    >>> text(fold(b'xyz'*24))
+    'xyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyzxyz'
     """
+    # 72 is the max line length we actually want, but the header field name
+    # has to fit in the first line too (See Debian Bug #863690).
+    maxleng = 72 - namelen
+    if len(header) <= maxleng:
+        return header
+    if len(header) - header.rfind(b"\r\n") == 2 and len(header) <= maxleng +2:
+        return header
     i = header.rfind(b"\r\n ")
     if i == -1:
         pre = b""
@@ -360,9 +371,6 @@ def fold(header, namelen=0):
         i += 3
         pre = header[:i]
         header = header[i:]
-    # 72 is the max line length we actually want, but the header field name
-    # has to fit in the first line too (See Debian Bug #863690).
-    maxleng = 72 - namelen
     while len(header) > maxleng:
         i = header[:maxleng].rfind(b" ")
         if i == -1:
@@ -372,7 +380,13 @@ def fold(header, namelen=0):
         pre += header[:j] + b"\r\n "
         header = header[j:]
         namelen = 0
-    return pre + header
+    if len(header) > 2:
+        return pre + header
+    else:
+        if pre[0] == b' ':
+            return pre[:-1]
+        else:
+            return pre + header
 
 def load_pk_from_dns(name, dnsfunc=get_txt):
   s = dnsfunc(name)
@@ -384,6 +398,12 @@ def load_pk_from_dns(name, dnsfunc=get_txt):
       pub = parse_tag_value(s)
   except InvalidTagValueList as e:
       raise KeyFormatError(e)
+  try:
+      if pub[b'v'] != b'DKIM1':
+          raise KeyFormatError("bad version")
+  except KeyError as e:
+      # Version not required in key record: RFC 6376 3.6.1
+      pass
   try:
       if pub[b'k'] == b'ed25519':
           pk = nacl.signing.VerifyKey(pub[b'p'], encoder=nacl.encoding.Base64Encoder)
@@ -400,6 +420,8 @@ def load_pk_from_dns(name, dnsfunc=get_txt):
       except (TypeError,UnparsableKeyError) as e:
           raise KeyFormatError("could not parse public key (%s): %s" % (pub[b'p'],e))
       ktag = b'rsa'
+  if pub[b'k'] != b'rsa' and pub[b'k'] != b'ed25519':
+      raise KeyFormatError('unknown algorithm in k= tag: {0}'.format(pub[b'k']))
   return pk, keysize, ktag
 
 #: Abstract base class for holding messages and options during DKIM/ARC signing and verification.
@@ -702,6 +724,12 @@ class DKIM(DomainSigner):
 
     if include_headers is None:
         include_headers = self.default_sign_headers()
+    try:
+        include_headers = [bytes(x, 'utf-8') for x in include_headers]
+    except TypeError:
+        # TypeError means it's already bytes and we're good or we're in
+        # Python 2 and we don't care.  See LP: #1776775.
+        pass
 
     include_headers = tuple([x.lower() for x in include_headers])
     # record what verify should extract
